
/********************/
/* tree nodes types */
/********************/

enum {
	ERROR,
	EMPTY_CONTEXTS_START,
	EMPTY_CONTEXTS_END,
	EMPTY_CONTEXTS_START_END,
	EMPTY_CONTEXTS,
	MARKUP_CONTEXT,
	LONGEST_MARKUP,
	LONGEST_MARKUP_RL,
	SHORTEST_MARKUP,
	SHORTEST_MARKUP_RL,
	LONGEST_REPLACE,
	LONGEST_REPLACE_RL,
	SHORTEST_REPLACE,
	SHORTEST_REPLACE_RL,
	NO_CONTEXT,
	OR_OR,
	SL_SL,
	BSL_BSL,
	BSL_SL,
	BEGIN_CODE,
	REGEX,
	DEFINE,
	REMARK,
	CODE,
	TERM_COMPLEMENT,
	COMPLEMENT,
	KLEENE_PLUS,
	KLEENE_STAR,
	CONTAINMENT,
	ONE_CONTAINMENT,
	MAX_ONE_CONTAINMENT,
	CONCATENATION_EXPR,
	CONCATENATION_REL,
	N_CONCATENATION,
	N_K_CONCATENATION,
	MORE_N_CONCATENATION,
	LESS_N_CONCATENATION,
	UNION_EXPR,
	INTERSECTION_EXPR,
	UNION_REL,
	INTERSECTION_REL,
	MINUS,
	IGNORE,
	INTERNAL_IGNORE,
	OPTION,
	ORDER,
	BEFORE,
	AFTER,
	SHUFFLE,
	REVERSE,
	DOMAIN,
	RANGE,
	RESTRICT,
	CROSSPRODUCT,
	COMPOSITION,
	PAIR,
	INVERT,
	REPLACE,
	REPLACE_OPTION,
	REPLACE_DOUBLE,
	REPLACE_DOUBLE_OPTION,
	REPLACE_REVERSE,
	REPLACE_REVERSE_OPTION,
	MARKUP,
	CONTEXT_START,
	CONTEXT_END,
	CONTEXT_START_END,
	CONTEXT,
	RIGHT_CONTEXT_START,
	RIGHT_CONTEXT_END,
	RIGHT_CONTEXT_START_END,
	RIGHT_CONTEXT,
	LEFT_CONTEXT_START,
	LEFT_CONTEXT_END,
	LEFT_CONTEXT_START_END,
	LEFT_CONTEXT,
	MERGE_CONTEXT,
	RULE,
	MERGE_RULE,
	REPLACE_RULE,
	REPLACE_RULE_CONTEXT,
	MERGE_REPLACE,
	RESTRICT_RULE,
	CHAR_CONCATENATION,
	OTHER_NO_QUOTES,
	OTHER_QUOTES,
	BOUND,
	ANY_SYMBOL,
	ONE_SYMBOL,
	EPSILON,
	INVERT_EXPR
};

/************/
/* typedefs */
/************/

/* tree  node structure */

typedef struct node {    
	int op;						  /* type of operation */
    char *str;					  /* hold names of identfiers*/
    int children;				  /* number of children (0,1 or 2) */
	struct node *child1,*child2;  /* pointers to children */
} *NODE;

/* structure keeping part of tree to be later parsed into macro */

typedef struct macro {    
	NODE r;						  /* the root of this part fo tree */
    int num;					  /* the number	of this macro */
	struct macro *next;			  /* pointer to next */
} *MACRO;

/*********************/
/* global variables: */
/*********************/

MACRO head;						 /* pointer to the head of macros list */
MACRO tail;						 /* pointer to the tail of macros list */
int index;						 /* number of macro in the macros list */
extern NODE root;				 /* root of the grammar tree */
int unsolved;					 /* count operators that can not be parsed */

/***********************/
/* functions prototpes */
/***********************/

void getTree(char *codeFile);
void print_tree(NODE r);
char *print_op(int op);
void print_fsa(NODE r);
void print_char_concat(char *str);
void print_no_quotes(char *str);
void replace_match(NODE r);
void replace(NODE r);
void MakeMacro(NODE r,int num);
void PrintReplace (int op);
void PrintMacro (void);
void PrintContextRule(NODE r);
void PrintParallelRules(MACRO m);
int FindNumRules(NODE r);
int ReplaceParallel(MACRO m);
int ReplaceOptionParallel(MACRO m);
int ReverseParallel(MACRO m);
int ReverseOptionParallel(MACRO m);
int ReplaceDoubleParallel(MACRO m);
void PrintUL(MACRO m,int num);
void PrintContextRuleReplace (NODE r);
void PrintLeft(NODE r);
void PrintRight(NODE r);
void PrintContextRuleReplaceOption (NODE r);
void PrintContextRuleReplaceReverse (NODE r);
void PrintContextRuleReplaceReverseOption (NODE r);
void free_tree(NODE r);
void free_macro(void);
void PrintStart(NODE r);
void PrintEnd(NODE r);
void SetValues(void);
void PrintEnd(NODE r);
void PrintStart(NODE r);
