%{ 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "typedef.h"
#include "xfst2fsa_tab.h"  

extern int line_number;

yywrap() {return 1; }

%}

REMARK ^"!"(.*)\n
CLEAR_STACK ^"clear stack"(.*)\n
MINIMIZE ^"minimize"(.*)\n
ECHO ^"echo"(.*)\n
DEFINE ^("define ")(([0-9a-zA-Z]*)[a-zA-Z]([0-9a-zA-Z]*))" "
RESERVE "0"|"`"|"~"|"!"|"@"|"#"|"$"|"%"|"^"|"&"|"*"|"("|")"|"-"|"_"|"="|"+"|"["|"]"|";"|":"|\"|","|"<"|"."|">"|"/"|"?"|" "
RESERVE_BUT_SPACE "0"|"`"|"~"|"!"|"@"|"#"|"$"|"%"|"^"|"&"|"*"|"("|")"|"-"|"_"|"="|"+"|"["|"]"|";"|":"|\"|","|"<"|"."|">"|"/"|"?"
ALL [a-zA-Z1-9]|{RESERVE}		
ALL_BUT_SPACE [a-zA-Z1-9]|{RESERVE_BUT_SPACE}		
LETTER			[a-zA-Z]
DIGIT			[1-9]
DIGIT_ZERO		[0-9]
NO_QUOTES [a-zA-Z1-9]|"0"|"`"|"~"|"!"|"@"|"#"|"$"|"%"|"^"|"&"|"*"|"("|")"|"-"|"_"|"="|"+"|"["|"]"|";"|":"|","|"<"|"."|">"|"/"|"?"|" "
CHAR_CONCAT "{"({NO_QUOTES}+)"}"
ONE {LETTER}|{DIGIT}|("%"{ALL})
EPS	0|(\[[ \t]*\])|(\"\")
ELSE_QUOTES \"({NO_QUOTES}+)\"
ELSE_NO_QUOTES ((\%{ALL})|{DIGIT_ZERO}|{LETTER})(((\%{ALL})|{DIGIT_ZERO}|{LETTER})+)
N_CONCAT "^"({DIGIT_ZERO}+)
N__K_CONCAT "^{"({DIGIT}|({DIGIT_ZERO}({DIGIT_ZERO}+)))","({DIGIT}|({DIGIT_ZERO}({DIGIT_ZERO}+)))"}"
MORE_CONCAT	"^>"({DIGIT}|({DIGIT_ZERO}({DIGIT_ZERO}+)))
LESS_CONCAT	"^<"({DIGIT}|({DIGIT_ZERO}({DIGIT_ZERO}+)))

%%
{DEFINE}	{ yylval.string = (char*) malloc(strlen(yytext)+1);
			  strcpy(yylval.string,yytext);	return(T_DEFINE); }
^"read regex"	{ return (T_REGEX); }
^"regex"	{ return (T_REGEX); }
";"([ \t]*)\n	{ line_number++; return (T_END); }
{REMARK}	{ line_number++; yylval.string = (char*) malloc(strlen(yytext)+1);
			  strcpy(yylval.string,yytext);	return(T_REMARK); }	
{CLEAR_STACK}	{ line_number++; yylval.string = (char*) malloc(strlen(yytext)+2);
			  strcpy(yylval.string,"!"); strcpy(yylval.string+1,yytext);	return(T_REMARK); }	
{MINIMIZE}	{ line_number++; yylval.string = (char*) malloc(strlen(yytext)+2);
			  strcpy(yylval.string,"!"); strcpy(yylval.string+1,yytext);	return(T_REMARK); }	
{ECHO}	{ line_number++; yylval.string = (char*) malloc(strlen(yytext)+2);
			  strcpy(yylval.string,"!"); strcpy(yylval.string+1,yytext);	return(T_REMARK); }	
{CHAR_CONCAT}	{ yylval.string = (char*) malloc(strlen(yytext)+1);
				strcpy(yylval.string,yytext);	return(T_CHAR_CONCATENATION); }	
"?"			{ yylval.string = (char*) malloc(strlen(yytext)+1);
			  strcpy(yylval.string,yytext);	return(T_ANY_SYMBOL); }	
{ONE}		{ yylval.string = (char*) malloc(strlen(yytext)+1);
			  strcpy(yylval.string,yytext);	return(T_ONE_SYMBOL); }	
{EPS}		{ yylval.string = (char*) malloc(strlen(yytext)+1);
			  strcpy(yylval.string,yytext);	return(T_EPSILON); }	
{ELSE_QUOTES} { yylval.string = (char*) malloc(strlen(yytext)+1);
			  strcpy(yylval.string,yytext);  return(T_OTHER_QUOTES); }	
{ELSE_NO_QUOTES} { yylval.string = (char*) malloc(strlen(yytext)+1);
			  strcpy(yylval.string,yytext);  return(T_OTHER_NO_QUOTES); }	
{N_CONCAT}	{ yylval.string = (char*) malloc(strlen(yytext)+1);
			  strcpy(yylval.string,yytext); return(T_N_CONCATENATION); }
{N__K_CONCAT}	{ yylval.string = (char*) malloc(strlen(yytext)+1);
			  strcpy(yylval.string,yytext); return(T_N_K_CONCATENATION); }
{MORE_CONCAT}	{ yylval.string = (char*) malloc(strlen(yytext)+1);
			  strcpy(yylval.string,yytext); return(T_MORE_N_CONCATENATION); }
{LESS_CONCAT}	{ yylval.string = (char*) malloc(strlen(yytext)+1);
			  strcpy(yylval.string,yytext); return(T_LESS_N_CONCATENATION); }
\\			{ return T_BACKSLASH; }
"~"			{ return T_TILDE; }
"+"			{ return T_PLUS; }
"*"			{ return T_ASTERISK; }
"$"			{ return T_DOLAR; }
"$."		{ return T_DOLAR_PERIOD; }
"$"([ \t]*)"?" { return T_DOLAR_QUESTION; }
"|"			{ return T_OR; }
"&"			{ return T_AND; }
"-"			{ return T_MINUS; }
"/"			{ return T_SLASH; }
"./."		{ return T_SLASH_PERIOD; }
"("			{ return T_LEFT_ROUND_BRACKET; }
")"			{ return T_RIGHT_ROUND_BRACKET; }
"["			{ return T_LEFT_RECT_BRACKET; }
"]"			{ return T_RIGHT_RECT_BRACKET; }
"<"			{ return T_LESS; }
">"			{ return T_MORE; }
".r"		{ return T_REVERSE; }
".x."		{ return T_CROSSPRODUCT; }
".o."		{ return T_COMPOSITION; }
":"			{ return T_COLON; }
".u"|".1"	{ return T_DOMAIN; }
".l"|".2"	{ return T_RANGE; }
".i"		{ return T_INVERT; }
"_"			{ return T_UNDERSCORE; }
"..."		{ return T_MARKUP; }
","			{ return T_COMMA; }
"@->"		{ return T_LONGEST_REPLACE; }
"@>"		{ return T_SHORTEST_REPLACE; }
"->@"		{ return T_LONGEST_REPLACE_RL; }
">@"		{ return T_SHORTEST_REPLACE_RL; }
"->"		{ return T_REPLACE; }
"(->)"		{ return T_OPTION_REPLACE; }
"<-"		{ return T_REV_REPLACE; }
"(<-)"		{ return T_OPTION_REV_REPLACE; }
"<->"		{ return T_DOUBLE_REPLACE; }
"(<->)"		{ return T_OPTION_DOUBLE_REPLACE; }
"||"		{ return T_TWO_OR; }
\\\\		{ return T_TWO_BACKSLASH; }
"//"		{ return T_TWO_SLASH; }
\\"/"		{ return T_BACKSLASH_SLASH; }
",,"		{ return T_DOUBLE_COMMA; }
"=>"		{ return T_RESTRICT; }
".#."|("["([ \t]*)".#."([ \t]*)"]")		{ return T_BOUND; }
"\n"		{ line_number++; }
([ \t]+)	;
.			{ fprintf(stdout,"ERROR: Unexpected char %c in line number %d.\n",yytext[0],line_number); }

%%

